#ifndef cdll_h_included
#define cdll_h_included

/* 
 * The following is gross, but needed to support both egcs-1.0.x, which
 * doesn't support dllimport/dllexport atttributes and newer egcs-1.1, 
 * which does.
 *
 * When egcs-1.0.x is dead, we can safely take out the "DISCARD" block:
 *  
 *   #ifndef DECLSPEC_NOT_SUPPORTED        |    |
 *     KEEP_THIS_CODE                      |    |
 *   #else / * DECLSPEC_NOT_SUPPORTED * /  | -> |  KEEP_THIS_CODE
 *     DISCARD_THIS_CODE                   |    |
 *   #endif / * DECLSPEC_NOT_SUPPORTED * / |    |
 *
 * And, completely kill code like the following:
 *
 *   #if BUILDING_DLL && defined(DECLSPEC_NOT_SUPPORT)
 *	COMPLETELY_DISTCARD_THIS_CODE
 *   #endif
 *
 * The upshot of the following set of definitions is that we have the
 * following definitions to be used by both DLL and client code:
 *
 * Building the DLL:
 *	- define BUILDING_DLL, which defines DLLIMPORT to be one of 
 *	  the following:
 *		egcs-1.0.x: <empty>
 *		egcs-1.1.x: __attribute__((dllexport))
 * Building the client code:
 *	- DO NOT define BUILDING_DLL, which defines DLLIMPORT to be one 
 *	  of the following:
 *		egcs-1.0.x: extern
 *		egcs-1.1.x: __attribute__((dllimport))
 */

#if BUILDING_DLL
# if __GNUC__ && ! defined(__declspec)
#  define DLLIMPORT
#  define DECLSPEC_NOT_SUPPORTED
# else /* ! __GNUC__ || __declspec */
#  define DLLIMPORT __declspec (dllexport)
# endif /* ! __GNUC__ || __declspec */
#else /* Not BUILDING_DLL */
# if __GNUC__ && ! defined(__declspec)
#  define DLLIMPORT extern
#  define DECLSPEC_NOT_SUPPORTED
# else /* ! __GNUC__ ||  __declspec) */
#  define DLLIMPORT __declspec (dllimport)
# endif /* ! __GNUC__ ||  __declspec) */
#endif /* Not BUILDING_DLL */

DLLIMPORT int dll_int_square (int) ;
DLLIMPORT double dll_double_square (double);
DLLIMPORT float dll_float_square (float);

#ifndef DECLSPEC_NOT_SUPPORTED

DLLIMPORT int dll_global_int_var;

#else /* DECLSPEC_NOT_SUPPORTED */

extern int (*_imp__dll_global_int_var);
#define dll_global_int_var (*_imp__dll_global_int_var)

#endif /* DECLSPEC_NOT_SUPPORTED */

DLLIMPORT void dll_set_global_int_var (int);
DLLIMPORT int dll_get_global_int_var ();

#endif /* cdll_h_included */

